//LineGraphView.cpp
// Copyright(C) 1999 by Kiyoshi Kobayashi. All rights reserved.
//
//

#undef DEBUG
#include <Point.h>
#include <OS.h>
#include <Message.h>
#include <MenuItem.h>
#include <PopUpMenu.h>
#include <Application.h>
#include <Debug.h>

#include "LineGraphView.h"
#include "MeterBaseView.h"
#include <stdio.h>
#include <string.h>
#include "MeterMessages.h"

static const char CLASS_NAME[] = "LineGraphView";
static const char VIEW_NAME[] = "LineGraphView";

static const float GAP = 1.0f;
static const float MARGIN = 1.0f;
static const rgb_color BORDER_COLOR = {200, 200, 200, 0};

// Public: Constructor.
//
LineGraphView::LineGraphView(BRect frame, const char *name, float maximum)
	: BView(frame, name, B_FOLLOW_ALL, B_WILL_DRAW|B_FRAME_EVENTS) {
	// Create ActualLineGraphView object
	view = new ActualLineGraphView(frame, "ActualLineGraphView", maximum);

	AddChild(view);
}

// Public: Constructor from the archive.
//
LineGraphView::LineGraphView(BMessage *archive)
	: BView(archive) {
	// restore transient pointer to the children
	//
	BView *child = this->ChildAt(0);

	while (child != NULL) {
		if (is_instance_of(child, ActualLineGraphView)) {
			view = cast_as(child, ActualLineGraphView);
			break;
		}
		child = child->NextSibling();
	}
	
	ASSERT(view != NULL);
}

// Public: Destructor.
//
LineGraphView::~LineGraphView() {}

// Public: Update the contents.
//
void
LineGraphView::Update(float value) {
	TRACE();
	view->Update(value);
}

// Public: Override BView method.
//
void
LineGraphView::AttachedToWindow() {
	TRACE();
	
	if (Parent()) {
		SetViewColor(Parent()->ViewColor());
		
		Adjust(Frame().Width(), Frame().Height());
		
		// redraw all lines
		RedrawAllLines(Frame().Width(), Frame().Height());
	} else
		TRESPASS();
	
	BView::AttachedToWindow();
}

// Public: Override BView method.
//
void
LineGraphView::FrameResized(float width, float height) {
	// redraw all lines
	RedrawAllLines(width, height);

	Adjust(width, height);
	
	BView::FrameResized(width, height);
}

// Public: Override BView method.
//
void
LineGraphView::Draw(BRect updateRect) {
	// redraw all lines
	RedrawAllLines(Frame().Width(), Frame().Height());
	
	BView::Draw(updateRect);
}

// Public: Archive the object.
//
status_t
LineGraphView::Archive(BMessage *archive, bool deep) const {
	status_t ret;
	ret = BView::Archive(archive, deep); 
	ASSERT(ret == B_OK);
	
	ret = archive->AddString("class", CLASS_NAME); 
	ASSERT(ret == B_OK);

	return B_OK;
}

// Public/Static: Instanciate the object.
//
BArchivable *
LineGraphView::Instantiate(BMessage *archive) {
	if (validate_instantiation(archive, CLASS_NAME))
		return new LineGraphView(archive); 
	
	return NULL; 
}

// Private:
//
LineGraphView::LineGraphView(const LineGraphView &)
	: BView(BRect(), NULL, B_FOLLOW_ALL, B_WILL_DRAW), view(NULL) {}

// Private: Draw border lines.
//
void
LineGraphView::RedrawAllLines(float width, float height) {
	// erase all
	FillRect(Bounds(), B_SOLID_LOW);
	
	// draw BORDER_COLOR
	BeginLineArray(2);

	AddLine(BPoint(0.0, height),
			BPoint(width, height), BORDER_COLOR);
	AddLine(BPoint(width, 0.0),
			BPoint(width, height), BORDER_COLOR);
	
	EndLineArray();
}

// Private: Adjust the size.
//
void
LineGraphView::Adjust(float width, float height) {
	view->MoveTo(MARGIN, MARGIN);
	view->ResizeTo(width - MARGIN * 2.0f,
					height - MARGIN * 2.0f);
}

